
/********************************************
 **** Core Class V1.1  1993-94 Yves Schmid & Alia Development
 ****
 **** CoreList.h
 ****
 **** Authors:		Yves Schmid and Odorico von Susani 
 **** Created:      14 November 1993
 **** Modified:     08 August 1994
 **** Compatible:   C++
 **** Version:      1.1
 **** Description:  CoreList is a class which handles linked list. The aim of
 ****				CoreList is to keep the trace of a linked list. A CoreList
 ****				object contains pointers on the first and on the last element 
 ****				of the list and a variable for the number of elements
 ****				stocked in the list.
 ****				
 ****				After a CoreList has been builded the list is empty. To add
 ****				an element to the CoreList object you must create a superclass
 ****				of CoreNode. You can add your CoreNode to the list with
 ****				the method "add" of the CoreList class. However using "add" is not
 ****				the smart way to add a node to a list. When possible use the
 ****				"setlist" method of the CoreNode class. The "setlist" method
 ****				checks if your node is already linked to another list and if
 ****				it is the case "setlist" unlinks your node from the old list
 ****				before linking the node to the new list. If you decide to
 ****				use the "add" method, be sure to verify that your node is
 ****				not linked to another list before adding it to a new one.
 ****
 ****				When you delete a CoreList, the destructor deletes all the
 ****				CoreNodes linked to his list.
 ****
 ****				When you duplicate a CoreList all CoreNodes are duplicated too.
 ****
 ****				If you send a command to a CoreList with the flag "CCF_NODES",
 ****				all the CoreNodes will receive the command.
 ****
 *************************/


#ifndef CoreList_H
#define CoreList_H

#include "Core.h"

class CoreHead;
class CoreNode;

class CoreList: public Core
{

	//***********************************************************
	//.............. P U B L I C   M E T H O D S.................
    
    public:

    CoreList(CoreHead *ch =NULL);	// Pass a CoreHead pointer if this CoreList is part
									// of a CoreHead structure. Most of the time you pass
									// NULL here.
    virtual ~CoreList(void);


    inline CoreNode *getfirst(void) const {return first;}	// Useful informations
    inline CoreNode *getlast(void) const {return last;}
    inline long getlength(void) const {return length;}


    virtual void docmd(long cmd,
                       short flags=0,
                       void *info=NULL,
                       short t=0,
                       void *parent=NULL); // The magic command system (see "core.h")

    virtual Core *duplicate(void);   // (see "core.h")


    virtual long find(const CoreNode *ln);	// Find position of a node in the list
    virtual CoreNode *find(const long p);	// Find a node by position


    virtual void swap(CoreNode *ln);       // Swap the node with the next node

	inline CoreHead *getitscorehead(void) const 	// NULL or its CoreHead if this CoreList is
		{return itscorehead;}				// part of a CoreHead object


	//***********************************************************

	//..........................................................
    // You should not call the following methods!

    inline void setfirst(CoreNode *n) {first=n;}	// Direct manipulation of
    inline void setlast(CoreNode *n) {last=n;}		// pointers, not recommanded!

    virtual void add(CoreNode *ln);	// Add a node (at the end of the list)

    virtual void add(CoreNode *ln, CoreNode *afterln);	// Add after a node
    inline void add(CoreNode *ln, long p) 				// Add after a position
    { add(ln,find(p)); }

    virtual void addstart(CoreNode *ln);             // Add a node at the start of the list

    virtual void remove(CoreNode *ln);		// Remove a node
    inline void remove(const long p) 		// Remove by position
    { remove(find(p)); }


    virtual void copyfrom(Core *uo); // The magic duplication system

	//..........................................................


	private:	

    CoreNode   *first,*last;
    long       length;

	CoreHead	*itscorehead;		// CoreList is part of a CoreHead structure or NULL

};



#endif
